<?php

namespace Drupal\cloud\Controller;

use Drupal\cloud\Traits\CloudContentEntityTrait;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityDeleteFormTrait;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller responsible for "update" URLs.
 *
 * This class is mainly responsible for updating the entities from URLs.
 */
class ApiController extends ControllerBase implements ApiControllerInterface {

  use CloudContentEntityTrait;
  use EntityDeleteFormTrait;

  /**
   * The entity used by this form.
   *
   * @var \Drupal\Core\Entity\ContentEntityInterface|\Drupal\Core\Entity\RevisionLogInterface
   */
  private $entity;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * ApiController constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Dependency Injection.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   Instance of ContainerInterface.
   *
   * @return ApiController
   *   return created object.
   */
  public static function create(ContainerInterface $container): ApiController {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Gets the entity of this form.
   *
   * Provided by \Drupal\Core\Entity\EntityForm.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   The entity.
   */
  public function getEntity(): EntityInterface {
    return $this->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function operateEntity(Request $request, string $cloud_service_provider, string $cloud_context, string $entity_type_id, string $entity_id, string $command): JsonResponse {
    // Create an instance of the entity.
    // The process splits between adding a new entity and when reading an
    // existing entity.
    /** @var \Drupal\cloud\Entity\CloudLaunchTemplateInterface $entity */
    $entity = $command !== 'create'
      ? $this->entityTypeManager
        ->getStorage($entity_type_id)
        ->load($entity_id)
      : $this->entityTypeManager
        ->getStorage($entity_type_id)
        ->create([
          'cloud_context' => $cloud_context,
        ]);
    if (empty($entity)) {
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'The entity has already been deleted.',
      ], 404);
    }

    if ($command !== 'delete') {
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'Not Found',
      ], 404);
    }

    try {
      $this->entity = $entity;
      $entity->delete();

      $this->messenger->addStatus($this->getDeletionMessage());
      $this->logDeletionMessage();

      // Clear block and menu cache.
      $this->clearCacheValues($this->entity->getCacheTags());
      return new JsonResponse([
        'result' => 'OK',
      ], 200);
    }
    catch (\Exception $e) {
      $this->handleException($e);
      return new JsonResponse([
        'result' => 'NG',
        'reason' => 'Internal Server Error',
      ], 500);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getImageOptions($cloud_service_provider, $cloud_context): JsonResponse {
    /** @var \Drupal\aws_cloud\Entity\Ec2\ImageInterface[] $images */
    $images = $this->entityTypeManager
      ->getStorage("{$cloud_service_provider}_image")
      ->loadByProperties([
        'cloud_context' => $cloud_context,
      ]);

    $options = [];
    foreach ($images as $image) {
      $options[$image->getImageId()] = sprintf('%s (%s)',
        $image->getName(),
        $image->getImageId()
      );
    }

    natcasesort($options);

    $output = [];
    foreach ($options ?: [] as $value => $label) {
      $output[] = [
        'value' => $value,
        'label' => $label,
      ];
    }
    return new JsonResponse($output, 200);
  }

  /**
   * {@inheritdoc}
   */
  public function getSecurityGroupOptions($cloud_service_provider, $cloud_context, $subnet_id = ''): JsonResponse {
    if ($cloud_service_provider === 'openstack') {
      $entities = $this->entityTypeManager
        ->getStorage('openstack_security_group')
        ->loadByProperties([
          'cloud_context' => $cloud_context,
        ]);

      $options = [];
      foreach ($entities ?: [] as $entity) {
        $options[$entity->id()] = $entity->getName();
      }

      natcasesort($options);

      $output = array_map(static function ($value, $label) {
        return [
          'value' => $value,
          'label' => $label,
        ];
      }, array_keys($options ?: []), array_values($options ?: []));
      return new JsonResponse($output, 200);
    }

    $vpc_id = NULL;
    if (!empty($subnet_id)) {
      /** @var \Drupal\aws_cloud\Entity\Vpc\SubnetInterface[] $subnets */
      $subnets = $this->entityTypeManager
        ->getStorage("{$cloud_service_provider}_subnet")
        ->loadByProperties([
          'cloud_context' => $cloud_context,
          'subnet_id' => $subnet_id,
        ]);

      if (!empty($subnets)) {
        $vpc_id = array_shift($subnets)->getVpcId();
      }
    }

    $storage = $this->entityTypeManager
      ->getStorage("{$cloud_service_provider}_security_group");

    $query = $storage
      ->getQuery()
      ->accessCheck(TRUE)
      ->condition('cloud_context', $cloud_context);

    if ($vpc_id !== NULL) {
      $query = $query->condition('vpc_id', $vpc_id);
    }

    $entity_ids = $query->execute();

    $options = [];
    $security_groups = $storage->loadMultiple($entity_ids);
    foreach ($security_groups ?: [] as $security_group) {
      $options[$security_group->id()] = $security_group->getGroupName();
    }

    natcasesort($options);

    $output = array_map(static function ($value, $label) {
      return [
        'value' => $value,
        'label' => $label,
      ];
    }, array_keys($options ?: []), array_values($options ?: []));
    return new JsonResponse($output, 200);
  }

  /**
   * {@inheritdoc}
   */
  public function getKeyPairOptions($cloud_service_provider, $cloud_context): JsonResponse {
    /** @var \Drupal\aws_cloud\Entity\Ec2\KeyPairInterface[] $keyPairs */
    $keyPairs = $this->entityTypeManager
      ->getStorage("{$cloud_service_provider}_key_pair")
      ->loadByProperties([
        'cloud_context' => $cloud_context,
      ]);

    $options = [];
    foreach ($keyPairs as $keyPair) {
      $options[$keyPair->id()] = $keyPair->getName();
    }

    natcasesort($options);

    $output = [];
    foreach ($options ?: [] as $value => $label) {
      $output[] = [
        'value' => $value,
        'label' => $label,
      ];
    }
    return new JsonResponse($output, 200);
  }

  /**
   * {@inheritdoc}
   */
  public function getNetworkOptions($cloud_service_provider, $cloud_context): JsonResponse {
    $networks = $this->entityTypeManager
      ->getStorage("{$cloud_service_provider}_network")
      ->loadByProperties([
        'cloud_context' => $cloud_context,
      ]);

    $options = [];
    foreach ($networks as $network) {
      $options[$network->id()] = $network->getName();
    }

    natcasesort($options);

    $output = [];
    foreach ($options ?: [] as $value => $label) {
      $output[] = [
        'value' => $value,
        'label' => $label,
      ];
    }
    return new JsonResponse($output, 200);
  }

}
